#!/usr/bin/python3

# This short script makes sure that any wheel that we produce
# will meet the 'manylinux' requirements of PEP513.
#
# For example, called 'check_manylinux_package show dist/pycifrw-4.2.1-cp35-cp35mu_linux_x86-64.whl'
#
from auditwheel import main
main.main()



#  Run files in the list, and catch any expected exceptions
from __future__ import print_function
from __future__ import absolute_import

import sys
#sys.path.insert(0,"..")
from CifFile import ReadCif,CifFile

def runtests(scantype,calling_method):
    """Scantype is flex or standard, calling method is either
    to construct a CifFile object or using ReadCif"""
    test_table = [
    ["ciftest1", "OK"],
    ["ciftest2", "OK"],
    ["ciftest3", "OK"],
    ["ciftest4", "OK"],
    ]

    for filename, testresult in test_table:
        try:
            calling_method(filename,scantype=scantype,grammar="2.0")
        except:
            stype,svalue,sinfo = sys.exc_info()
            if testresult == 'OK':
                print("%s causes error where none expected" % filename)
                print("%s\n%s" % (repr(stype),svalue))
            else:
                if stype in testresult:
                    print("%s passes" % filename)
                else:
                    print("Unexpected exception %s for %s" % (repr(stype),filename))
        else:     #no exception
            if testresult == 'OK':
                print("%s passes" % filename)
            else:
                print("%s: Expected %s, got nothing" % (filename,repr(testresult)))
if __name__ == "__main__":
    print("Testing interpreted tokenizer+ReadCif")
    runtests("standard",ReadCif)
    print("Testing compiled tokenizer+ReadCif",ReadCif)
    runtests("flex",ReadCif)
    print("Testing interpreted tokenizer+CifFile")
    runtests("standard",CifFile)
    print("Testing compiled tokenizer+CifFile")
    runtests("flex",CifFile)



#  Run files in the list, and catch any expected exceptions
from __future__ import absolute_import
from __future__ import print_function

import sys
from CifFile import ReadCif,CifError
from CifFile.StarFile import StarError

def runtests(scantype):
    test_table = [
    ["ciftest1", "OK"],
    ["ciftest2", "OK"],
    ["ciftest3", "OK"],
    ["ciftest4", "OK"],
    ["ciftest5", "OK"],
    ["ciftest6", [CifError,StarError]],
    ["ciftest7", [CifError,StarError]],
    ["ciftest8", [CifError,StarError]],
    ["ciftest9", [CifError,StarError]],
    ["ciftest10", [CifError,StarError]],
    ["ciftest11", "OK"],
    ["ciftest12", "OK"],
    ["ciftest13", "OK"],
    ["ciftest14", [CifError,StarError]],
    ["ciftest15", [CifError,StarError]],
    ["ciftest16", [CifError,StarError]],
    ["ciftest17", [CifError,StarError]],
    ["ciftest18", [CifError,StarError]]
    ]

    for filename, testresult in test_table:
        try:
            ReadCif(filename,scantype=scantype)
        except:
            stype,svalue,ss = sys.exc_info()
            if testresult == 'OK':
                print("%s causes error where none expected" % filename)
                print("%s\n%s" % (repr(stype),svalue))
            else:
                if stype in testresult:
                    print("%s passes" % filename)
                else:
                    print("Unexpected exception %s for %s" % (repr(stype),filename))
        else:     #no exception
            if testresult == 'OK':
                print( "%s passes" % filename)
            else:
                print( "%s: Expected %s, got nothing" % (filename,repr(testresult)))
if __name__ == "__main__":
    print("Testing interpreted tokenizer")
    runtests("standard")
    print("Testing compiled tokenizer")
    runtests("flex")



#!/usr/bin/python2

#
# Yapps 2 - yet another python parser system
# Copyright 1999-2003 by Amit J. Patel <amitp@cs.stanford.edu>
#
# This version of Yapps 2 can be distributed under the
# terms of the MIT open source license, either found in the LICENSE file
# included with the Yapps distribution
# <http://theory.stanford.edu/~amitp/yapps/> or at
# <http://www.opensource.org/licenses/mit-license.php>
#

from __future__ import print_function

import sys, re

import yappsrt, parsetree

def generate(inputfilename, outputfilename='', dump=0, **flags):
    """Generate a grammar, given an input filename (X.g)
    and an output filename (defaulting to X.py)."""

    if not outputfilename:
        if inputfilename.endswith('.g'):
            outputfilename = inputfilename[:-2] + '.py'
        else:
            raise Exception('Must specify output filename if input filename is not *.g')

    DIVIDER = '\n%%\n' # This pattern separates the pre/post parsers
    preparser, postparser = None, None # Code before and after the parser desc

    # Read the entire file
    s = open(inputfilename,'r').read()

    # See if there's a separation between the pre-parser and parser
    f = s.find(DIVIDER)
    if f >= 0: preparser, s = s[:f]+'\n\n', s[f+len(DIVIDER):]

    # See if there's a separation between the parser and post-parser
    f = s.find(DIVIDER)
    if f >= 0: s, postparser = s[:f], '\n\n'+s[f+len(DIVIDER):]

    # Create the parser and scanner and parse the text
    scanner = grammar.ParserDescriptionScanner(s)
    if preparser: scanner.first_line_number = 1 + preparser.count('\n')
    parser = grammar.ParserDescription(scanner)
    t = yappsrt.wrap_error_reporter(parser, 'Parser')
    if t is None: return # Failure
    if preparser is not None: t.preparser = preparser
    if postparser is not None: t.postparser = postparser

    # Check the options
    for f in t.options.keys():
        for opt,_,_ in yapps_options:
            if f == opt: break
        else:
            print('Warning: unrecognized option', f, file=sys.stderr)
    # Add command line options to the set
    for f in flags.keys(): t.options[f] = flags[f]

    # Generate the output
    if dump:
        t.dump_information()
    else:
        t.output = open(outputfilename, 'w')
        t.generate_output()

if __name__ == '__main__':
    import doctest
    doctest.testmod(sys.modules['__main__'])
    doctest.testmod(parsetree)

    # Someday I will use optparse, but Python 2.3 is too new at the moment.
    yapps_options = [
        ('context-insensitive-scanner',
         'context-insensitive-scanner',
         'Scan all tokens (see docs)'),
        ]

    import getopt
    optlist, args = getopt.getopt(sys.argv[1:], 'f:', ['help', 'dump', 'use-devel-grammar'])
    if not args or len(args) > 2:
        print('Usage:', file=sys.stderr)
        print('  python', sys.argv[0], '[flags] input.g [output.py]', file=sys.stderr)
        print('Flags:', file=sys.stderr)
        print(('  --dump' + ' '*40)[:35] + 'Dump out grammar information', file=sys.stderr)
        print(('  --use-devel-grammar' + ' '*40)[:35] + 'Use the devel grammar parser from yapps_grammar.py instead of the stable grammar from grammar.py', file=sys.stderr)
        for flag, _, doc in yapps_options:
            print(('  -f' + flag + ' '*40)[:35] + doc, file=sys.stderr)
    else:
        # Read in the options and create a list of flags
        flags = {}
        use_devel_grammar = 0
        for opt in optlist:
            for flag, name, _ in yapps_options:
                if opt == ('-f', flag):
                    flags[name] = 1
                    break
            else:
                if opt == ('--dump', ''):
                    flags['dump'] = 1
                elif opt == ('--use-devel-grammar', ''):
                    use_devel_grammar = 1
                else:
                    print('Warning: unrecognized option', opt[0], opt[1], file=sys.stderr)

        if use_devel_grammar:
            import yapps_grammar as grammar
        else:
            import grammar

        generate(*tuple(args), **flags)



# this simply reports time taken to parse a file
from __future__ import print_function
import time
from CifFile import ReadCif,Validate,CifDic
import cProfile
import pstats
testfiles = ['1wic.cif','ag03.cif','C13H22O3.cif','../dictionaries/cif_core.dic',
              '../dictionaries/mmcif_pdbx.dic','img2cif_packed.cif']
#cached_mmdic = CifDic('../dictionaries/mmcif_pdbx.dic',standard=None)
#cached_mmdic = CifDic('../dictionaries/mmcif_std_2.0.09.dic',standard=None)
cached_core = CifDic('../dictionaries/cif_core.dic')
#valfiles = [ ('ag03.cif',cached_mmdic),
#             ('C13H22O3.cif',cached_core)
valfiles = [
           ]

for file in testfiles:
    start_time = time.time()
    jj = ReadCif(file,scantype='flex',standard=None)      # no profiling, approx usage
    finish_time = time.time()
    print("File %s: wallclock time %8.1f\n" % (file,finish_time - start_time))
    cProfile.run("jj = ReadCif(file,scantype='flex',standard=None) ","profout")
    p = pstats.Stats( "profout")
    p.strip_dirs().sort_stats("cumulative").print_stats()
    # try to validate
for file,dic in valfiles:
    start_time = time.time()
    jj = Validate(file,dic=dic)
    finish_time = time.time()
    print("Validate file %s: %8.1f\n" % (file,finish_time - start_time))



#!/usr/bin/python2
#
# grammar.py, part of Yapps 2 - yet another python parser system
# Copyright 1999-2003 by Amit J. Patel <amitp@cs.stanford.edu>
#
# This version of the Yapps 2 grammar can be distributed under the
# terms of the MIT open source license, either found in the LICENSE
# file included with the Yapps distribution
# <http://theory.stanford.edu/~amitp/yapps/> or at
# <http://www.opensource.org/licenses/mit-license.php>
#

"""Parser for Yapps grammars.

This file defines the grammar of Yapps grammars.  Naturally, it is
implemented in Yapps.  The grammar.py module needed by Yapps is built
by running Yapps on yapps_grammar.g.  (Holy circularity, Batman!)

"""

import sys, re
import parsetree

######################################################################
def cleanup_choice(rule, lst):
    if len(lst) == 0: return Sequence(rule, [])
    if len(lst) == 1: return lst[0]
    return parsetree.Choice(rule, *tuple(lst))

def cleanup_sequence(rule, lst):
    if len(lst) == 1: return lst[0]
    return parsetree.Sequence(rule, *tuple(lst))

def resolve_name(rule, tokens, id, args):
    if id in [x[0] for x in tokens]:
        # It's a token
        if args:
            print('Warning: ignoring parameters on TOKEN %s<<%s>>' % (id, args))
        return parsetree.Terminal(rule, id)
    else:
        # It's a name, so assume it's a nonterminal
        return parsetree.NonTerminal(rule, id, args)


# Begin -- grammar generated by Yapps
import sys, re
import yapps3_compiled_rt as yappsrt

class ParserDescriptionScanner(yappsrt.Scanner):
    patterns = [
        ('"rule"', re.compile('rule')),
        ('"ignore"', re.compile('ignore')),
        ('"token"', re.compile('token')),
        ('"option"', re.compile('option')),
        ('":"', re.compile(':')),
        ('"parser"', re.compile('parser')),
        ('[ \t\r\n]+', re.compile('[ \t\r\n]+')),
        ('#.*?\r?\n', re.compile('#.*?\r?\n')),
        ('EOF', re.compile('$')),
        ('ATTR', re.compile('<<.+?>>')),
        ('STMT', re.compile('{{.+?}}')),
        ('STR', re.compile('[uUrR]?\'([^\\n\'\\\\]|\\\\.)*\'|[uUrR]?"([^\\n"\\\\]|\\\\.)*"')),
        ('ID', re.compile('[a-zA-Z_][a-zA-Z_0-9]*')),
        ('LP', re.compile('\\(')),
        ('RP', re.compile('\\)')),
        ('LB', re.compile('\\[')),
        ('RB', re.compile('\\]')),
        ('OR', re.compile('[|]')),
        ('STAR', re.compile('[*]')),
        ('PLUS', re.compile('[+]')),
        ('QUEST', re.compile('[?]')),
        ('COLON', re.compile(':')),
    ]
    def __init__(self, str):
        yappsrt.Scanner.__init__(self,None,['[ \t\r\n]+', '#.*?\r?\n'],str)

class ParserDescription(yappsrt.Parser):
    Context = yappsrt.Context
    def LINENO(self, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'LINENO', [])
        return 1 + self._scanner.get_input_scanned().count('\n')

    def Parser(self, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'Parser', [])
        self._scan('"parser"')
        ID = self._scan('ID')
        self._scan('":"')
        Options = self.Options(_context)
        Tokens = self.Tokens(_context)
        Rules = self.Rules(Tokens, _context)
        EOF = self._scan('EOF')
        return parsetree.Generator(ID,Options,Tokens,Rules)

    def Options(self, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'Options', [])
        opt = {}
        while self._peek() == '"option"':
            self._scan('"option"')
            self._scan('":"')
            Str = self.Str(_context)
            opt[Str] = 1
        if self._peek() not in ['"option"', '"token"', '"ignore"', 'EOF', '"rule"']:
            raise yappsrt.YappsSyntaxError(charpos=self._scanner.get_prev_char_pos(), context=_context, msg='Need one of ' + ', '.join(['"option"', '"token"', '"ignore"', 'EOF', '"rule"']))
        return opt

    def Tokens(self, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'Tokens', [])
        tok = []
        while self._peek() in ['"token"', '"ignore"']:
            _token = self._peek()
            if _token == '"token"':
                self._scan('"token"')
                ID = self._scan('ID')
                self._scan('":"')
                Str = self.Str(_context)
                tok.append( (ID,Str) )
            elif _token == '"ignore"':
                self._scan('"ignore"')
                self._scan('":"')
                Str = self.Str(_context)
                tok.append( ('#ignore',Str) )
            else:
                raise yappsrt.YappsSyntaxError(_token[0], 'Could not match Tokens')
        if self._peek() not in ['"token"', '"ignore"', 'EOF', '"rule"']:
            raise yappsrt.YappsSyntaxError(charpos=self._scanner.get_prev_char_pos(), context=_context, msg='Need one of ' + ', '.join(['"token"', '"ignore"', 'EOF', '"rule"']))
        return tok

    def Rules(self, tokens, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'Rules', [tokens])
        rul = []
        while self._peek() == '"rule"':
            LINENO = self.LINENO(_context)
            self._scan('"rule"')
            ID = self._scan('ID')
            OptParam = self.OptParam(_context)
            self._scan('":"')
            ClauseA = self.ClauseA(ID, tokens, _context)
            rul.append( (ID, OptParam, ClauseA) )
        if self._peek() not in ['"rule"', 'EOF']:
            raise yappsrt.YappsSyntaxError(charpos=self._scanner.get_prev_char_pos(), context=_context, msg='Need one of ' + ', '.join(['"rule"', 'EOF']))
        return rul

    def ClauseA(self, rule, tokens, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'ClauseA', [rule, tokens])
        ClauseB = self.ClauseB(rule, tokens, _context)
        v = [ClauseB]
        while self._peek() == 'OR':
            OR = self._scan('OR')
            ClauseB = self.ClauseB(rule, tokens, _context)
            v.append(ClauseB)
        if self._peek() not in ['OR', 'RP', 'RB', '"rule"', 'EOF']:
            raise yappsrt.YappsSyntaxError(charpos=self._scanner.get_prev_char_pos(), context=_context, msg='Need one of ' + ', '.join(['OR', 'RP', 'RB', '"rule"', 'EOF']))
        return cleanup_choice(rule, v)

    def ClauseB(self, rule, tokens, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'ClauseB', [rule, tokens])
        v = []
        while self._peek() in ['STR', 'ID', 'LP', 'LB', 'STMT']:
            ClauseC = self.ClauseC(rule, tokens, _context)
            v.append(ClauseC)
        if self._peek() not in ['STR', 'ID', 'LP', 'LB', 'STMT', 'OR', 'RP', 'RB', '"rule"', 'EOF']:
            raise yappsrt.YappsSyntaxError(charpos=self._scanner.get_prev_char_pos(), context=_context, msg='Need one of ' + ', '.join(['STR', 'ID', 'LP', 'LB', 'STMT', 'OR', 'RP', 'RB', '"rule"', 'EOF']))
        return cleanup_sequence(rule, v)

    def ClauseC(self, rule, tokens, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'ClauseC', [rule, tokens])
        ClauseD = self.ClauseD(rule, tokens, _context)
        _token = self._peek()
        if _token == 'PLUS':
            PLUS = self._scan('PLUS')
            return parsetree.Plus(rule, ClauseD)
        elif _token == 'STAR':
            STAR = self._scan('STAR')
            return parsetree.Star(rule, ClauseD)
        elif _token == 'QUEST':
            QUEST = self._scan('QUEST')
            return parsetree.Option(rule, ClauseD)
        elif _token not in ['"ignore"', '"token"', '"option"', '":"', '"parser"', 'ATTR', 'COLON']:
            return ClauseD
        else:
            raise yappsrt.YappsSyntaxError(_token[0], 'Could not match ClauseC')

    def ClauseD(self, rule, tokens, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'ClauseD', [rule, tokens])
        _token = self._peek()
        if _token == 'STR':
            STR = self._scan('STR')
            t = (STR, eval(STR,{},{}))
            if t not in tokens: tokens.insert( 0, t )
            return parsetree.Terminal(rule, STR)
        elif _token == 'ID':
            ID = self._scan('ID')
            OptParam = self.OptParam(_context)
            return resolve_name(rule, tokens, ID, OptParam)
        elif _token == 'LP':
            LP = self._scan('LP')
            ClauseA = self.ClauseA(rule, tokens, _context)
            RP = self._scan('RP')
            return ClauseA
        elif _token == 'LB':
            LB = self._scan('LB')
            ClauseA = self.ClauseA(rule, tokens, _context)
            RB = self._scan('RB')
            return parsetree.Option(rule, ClauseA)
        elif _token == 'STMT':
            STMT = self._scan('STMT')
            return parsetree.Eval(rule, STMT[2:-2])
        else:
            raise yappsrt.YappsSyntaxError(_token[0], 'Could not match ClauseD')

    def OptParam(self, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'OptParam', [])
        _token = self._peek()
        if _token == 'ATTR':
            ATTR = self._scan('ATTR')
            return ATTR[2:-2]
        elif _token not in ['"ignore"', '"token"', '"option"', '"parser"', 'COLON']:
            return ''
        else:
            raise yappsrt.YappsSyntaxError(_token[0], 'Could not match OptParam')

    def Str(self, _parent=None):
        _context = self.Context(_parent, self._scanner, self._pos, 'Str', [])
        STR = self._scan('STR')
        return eval(STR,{},{})


def parse(rule, text):
    P = ParserDescription(ParserDescriptionScanner(text))
    return yappsrt.wrap_error_reporter(P, rule)

# End -- grammar generated by Yapps



# Setup file for creation of the PyCIFRW
# distribution
from __future__ import print_function

from setuptools import setup, Extension, find_packages

#### Do the setup
c_scanner = Extension("CifFile.StarScan",
            sources = ["src/lib/lex.yy.c","src/lib/py_star_scan.c"])

setup(name="PyCifRW",
      version = "4.4.6",
      description = "CIF/STAR file support for Python",
      author = "James Hester",
      author_email = "jamesrhester@gmail.com",
      license = 'Python 2.0',
      url="https://github.com/jamesrhester/pycifrw/blob/development/README.md",
      classifiers = [
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: Python Software Foundation License',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 2',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
          'Topic :: Software Development :: Libraries :: Python Modules'
      ],
      py_modules = ['CifFile.CifFile_module','CifFile.yapps3_compiled_rt','CifFile.YappsStarParser_1_1','CifFile.YappsStarParser_1_0',
                    'CifFile.YappsStarParser_STAR2','CifFile.YappsStarParser_2_0','CifFile.StarFile','CifFile.TypeContentsParser'],
      ext_modules = [c_scanner],
      packages = ['CifFile', 'CifFile.drel'],
      test_suite = 'TestPyCIFRW',
      package_dir = {'CifFile':'src'}
      )
